@@ -12,8 +12,9 @@
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 //
-// $URL$
-// $Id$
+// $URL: https://github.com/CGAL/cgal/blob/releases/CGAL-4.14.3/Nef_S2/include/CGAL/Nef_polyhedron_S2.h $
+// $Id: Nef_polyhedron_S2.h 1f51c8a 2018-11-26T15:44:17+01:00 Andreas Fabri
+// SPDX-License-Identifier: GPL-3.0+
 // 
 //
 // Author(s)     : Michael Seel       <seel@mpi-sb.mpg.de>
@@ -24,11 +25,7 @@
 
 #include <CGAL/license/Nef_S2.h>
 
-
-#if defined(BOOST_MSVC)
-#  pragma warning(push)
-#  pragma warning(disable:4800) // complaint about performance in std::map where we can't do anything
-#endif
+#include <CGAL/disable_warnings.h>
 
 #include <CGAL/basic.h>
 #include <CGAL/Handle_for.h>
@@ -353,6 +350,11 @@ public:
             f->mark() == true);
   }
 
+  bool is_sphere() const
+  {
+    return is_plane();
+  }
+
   void extract_complement()
   { CGAL_NEF_TRACEN("extract complement");
     if ( this->is_shared() ) clone_rep();
@@ -378,7 +380,7 @@ public:
     SHalfedge_iterator e;
     CGAL_forall_svertices(v,D) v->mark() = false;
     CGAL_forall_sedges(e,D) e->mark() = false;
-    if ( D.has_sloop() ) D.shalfloop()->mark() = false;
+    if ( D.has_shalfloop() ) D.shalfloop()->mark() = false;
     D.simplify();
   }
 
@@ -393,7 +395,7 @@ public:
     CGAL_forall_svertices(v,D) v->mark() = true;
     CGAL_forall_sedges(e,D)    e->mark() = true;
     CGAL_forall_sfaces(f,D)    f->mark() = false;
-    if ( D.has_sloop() )       D.shalfloop()->mark() = D.shalfoop()->twin() = true;
+    if ( D.has_shalfloop() )       D.shalfloop()->mark() = D.shalfoop()->twin()->mark() = true;
     D.simplify();
   }
 
@@ -728,10 +730,8 @@ std::istream& operator>>
   return is;
 }
 
-
-#if defined(BOOST_MSVC)
-#  pragma warning(pop)
-#endif
-
 } //namespace CGAL
+
+#include <CGAL/enable_warnings.h>
+
 #endif //CGAL_NEF_POLYHEDRON_S2_H
